@@ -12,12 +12,12 @@ import io.mockk.every
 import io.mockk.mockk
 import io.mockk.verify
 import org.apache.commons.io.FileUtils
+import org.apache.commons.io.FilenameUtils
 import org.apache.http.entity.ContentType
 import org.jsoup.Connection
 import org.junit.jupiter.api.BeforeEach
 import org.junit.jupiter.api.Test
 import java.io.IOException
-import java.net.URI
 import java.nio.file.Path
 
 class SpongeTest {
@@ -27,16 +27,15 @@ class SpongeTest {
     private val imageFileName = "test.png"
 
     private val spongeInput = SpongeInput(
-            URI("https://test.com"),
-            outputDirectory,
-            setOf(ContentType.TEXT_PLAIN.mimeType),
-            setOf("png")
+        "https://test.com".toSpongeUri(),
+        outputDirectory,
+        setOf(ContentType.TEXT_PLAIN.mimeType),
+        setOf("png")
     )
 
     private val spongeInputWithSubdomains = spongeInput.copy(includeSubdomains = true)
     private val spongeInputWithDepthTwo = spongeInput.copy(maxDepth = 2)
 
-
     @BeforeEach
     fun beforeEach() {
         FileUtils.deleteDirectory(outputDirectory.toFile())
@@ -44,31 +43,31 @@ class SpongeTest {
 
     @Test
     fun testEmptyDocument() {
-        givenDocument(spongeInput.uri, "<html></html>")
+        givenDocument(spongeInput.spongeUri, "<html></html>")
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.download(any(), any()) }
     }
 
     @Test
     fun testUnsupportedDocument() {
-        givenDocument(spongeInput.uri, "", ContentType.IMAGE_TIFF)
+        givenDocument(spongeInput.spongeUri, "", ContentType.IMAGE_TIFF)
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.download(any(), any()) }
     }
 
     @Test
     fun testDocumentWithLink() {
-        val fileUri = URI("${spongeInput.uri}/$fileName")
+        val fileUri = "${spongeInput.spongeUri}/$fileName".toSpongeUri()
 
         givenDocument(
-                spongeInput.uri,
-                """
+            spongeInput.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$fileUri" />
@@ -81,19 +80,19 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, spongeInput.outputDirectory.resolve(fileName)) }
+        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
 
     @Test
     fun testDocumentWithLinkAndImage() {
-        val fileUri = URI("${spongeInput.uri}/$fileName")
-        val imageFileUri = URI("${spongeInput.uri}/$imageFileName")
+        val fileUri = "${spongeInput.spongeUri}/$fileName".toSpongeUri()
+        val imageFileUri = "${spongeInput.spongeUri}/$imageFileName".toSpongeUri()
 
         givenDocument(
-                spongeInput.uri,
-                """
+            spongeInput.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$fileUri" />
@@ -108,20 +107,20 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, spongeInput.outputDirectory.resolve(fileName)) }
+        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
         verify(exactly = 1) { spongeService.request(imageFileUri) }
-        verify(exactly = 1) { spongeService.download(imageFileUri, spongeInput.outputDirectory.resolve(imageFileName)) }
+        verify(exactly = 1) { spongeService.download(imageFileUri, getOutputFilePath(imageFileUri)) }
     }
 
     @Test
     fun testDocumentWithLinkAndSubdomainDisabled() {
-        val fileUri = URI("https://www.test.test.com/$fileName")
+        val fileUri = "https://www.test.test.com/$fileName".toSpongeUri()
 
         givenDocument(
-                spongeInput.uri,
-                """
+            spongeInput.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$fileUri" />
@@ -134,18 +133,18 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.request(fileUri) }
-        verify(exactly = 0) { spongeService.download(fileUri, spongeInput.outputDirectory.resolve(fileName)) }
+        verify(exactly = 0) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
 
     @Test
     fun testDocumentWithLinkAndSubdomainEnabled() {
-        val fileUri = URI("https://test.test.com/$fileName")
+        val fileUri = "https://test.test.com/$fileName".toSpongeUri()
 
         givenDocument(
-                spongeInputWithSubdomains.uri,
-                """
+            spongeInputWithSubdomains.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$fileUri" />
@@ -158,22 +157,22 @@ class SpongeTest {
 
         executeSponge(spongeInputWithSubdomains)
 
-        verify(exactly = 1) { spongeService.request(spongeInputWithSubdomains.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInputWithSubdomains.spongeUri) }
         verify(exactly = 1) { spongeService.request(fileUri) }
 
         verify(exactly = 1) {
-            spongeService.download(fileUri, spongeInputWithSubdomains.outputDirectory.resolve(fileName))
+            spongeService.download(fileUri, getOutputFilePath(fileUri))
         }
     }
 
     @Test
     fun testDocumentWithChildDocumentAndLink() {
-        val childDocumentUri = URI("https://test.com/test")
-        val fileUri = URI("${spongeInputWithDepthTwo.uri}/$fileName")
+        val childDocumentUri = "https://test.com/test".toSpongeUri()
+        val fileUri = "${spongeInputWithDepthTwo.spongeUri}/$fileName".toSpongeUri()
 
         givenDocument(
-                spongeInputWithDepthTwo.uri,
-                """
+            spongeInputWithDepthTwo.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$childDocumentUri" />
@@ -183,8 +182,8 @@ class SpongeTest {
         )
 
         givenDocument(
-                childDocumentUri,
-                """
+            childDocumentUri,
+            """
                     <html>
                         <body>
                             <a href="$fileUri" />
@@ -197,24 +196,66 @@ class SpongeTest {
 
         executeSponge(spongeInputWithDepthTwo)
 
-        verify(exactly = 1) { spongeService.request(spongeInputWithDepthTwo.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInputWithDepthTwo.spongeUri) }
         verify(exactly = 1) { spongeService.request(childDocumentUri) }
         verify(exactly = 1) { spongeService.request(fileUri) }
 
         verify(exactly = 1) {
-            spongeService.download(fileUri, spongeInputWithDepthTwo.outputDirectory.resolve(fileName))
+            spongeService.download(fileUri, getOutputFilePath(fileUri))
         }
     }
 
     @Test
-    fun testDocumentWithLinkAndFailedConnection() {
-        val fileUri = URI("${spongeInput.uri}/$fileName")
-        val failingFileName = "test2.txt"
-        val failingFileUri = URI("${spongeInput.uri}/$failingFileName")
+    fun testDocumentWithIgnoredChildDocument() {
+        val childDocumentUri = "https://test2.com".toSpongeUri()
 
         givenDocument(
-                spongeInput.uri,
+            spongeInput.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$childDocumentUri" />
+                        </body>
+                    </html>
                 """
+        )
+
+        executeSponge(spongeInput)
+
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify(exactly = 0) { spongeService.request(childDocumentUri) }
+    }
+
+    @Test
+    fun testDocumentWithInvalidChildUri() {
+        val childUri = "http://"
+
+        givenDocument(
+            spongeInput.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$childUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        executeSponge(spongeInput)
+
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify(exactly = 1) { spongeService.request(any()) }
+    }
+
+    @Test
+    fun testDocumentWithLinkAndFailedConnection() {
+        val fileUri = "${spongeInput.spongeUri}/$fileName".toSpongeUri()
+        val failingFileName = "test2.txt"
+        val failingFileUri = "${spongeInput.spongeUri}/$failingFileName".toSpongeUri()
+
+        givenDocument(
+            spongeInput.spongeUri,
+            """
                     <html>
                         <body>
                             <a href="$failingFileUri" />
@@ -230,36 +271,49 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.uri) }
+        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 1) { spongeService.request(failingFileUri) }
 
         verify(exactly = 0) {
-            spongeService.download(failingFileUri, spongeInput.outputDirectory.resolve(failingFileName))
+            spongeService.download(failingFileUri, getOutputFilePath(failingFileUri))
         }
 
         verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, spongeInput.outputDirectory.resolve(fileName)) }
+        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
 
-    private fun givenDocument(uri: URI, htmlContent: String, contentType: ContentType = ContentType.TEXT_HTML) {
+    private fun getOutputFilePath(spongeUri: SpongeUri): Path {
+        val uri = spongeUri.toUri()
+
+        return spongeInput.outputDirectory
+            .resolve(uri.host)
+            .resolve(FilenameUtils.getPath(uri.path))
+            .resolve(FilenameUtils.getName(uri.path))
+    }
+
+    private fun givenDocument(
+        spongeUri: SpongeUri,
+        htmlContent: String,
+        contentType: ContentType = ContentType.TEXT_HTML
+    ) {
         val response = mockk<Connection.Response>()
 
         every { response.contentType() } returns contentType.mimeType
         every { response.body() } returns htmlContent
-        every { response.url() } returns uri.toURL()
+        every { response.url() } returns spongeUri.toUri().toURL()
 
-        every { spongeService.request(uri) } returns response
+        every { spongeService.request(spongeUri) } returns response
     }
 
-    private fun givenFile(uri: URI, mimeType: String = ContentType.TEXT_PLAIN.mimeType) {
+    private fun givenFile(spongeUri: SpongeUri, mimeType: String = ContentType.TEXT_PLAIN.mimeType) {
         val response = mockk<Connection.Response>()
 
         every { response.contentType() } returns mimeType
-        every { spongeService.request(uri) } returns response
+        every { spongeService.request(spongeUri) } returns response
     }
 
-    private fun givenUriFailsConnection(uri: URI) {
-        every { spongeService.request(uri) } throws IOException()
+    private fun givenUriFailsConnection(spongeUri: SpongeUri) {
+        every { spongeService.request(spongeUri) } throws IOException("Error!")
     }
 
     private fun executeSponge(spongeInput: SpongeInput) {
